#include "mvUtil.h"
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <XSTRING>

void mvUtil::interp2d(float *x, float *y, float *z, float *dx, float *dy, float *zc,
			int numRow, int numCol, float znull)
{
	int i, j, cellOffset, pointOffset;
	float w, v;
	int nx = numCol + 1;
	int ny = numRow + 1;
	int nxy = nx*ny;

	// Set up arrays for interpolation
	float *value = new float [nxy];
	float *weight = new float [nxy];
	for (i=0; i<nxy; i++)
	{
		value[i] = 0;
		weight[i] = 0;
	}

	// Do inverse distance interpolation
	for (j=0; j<numRow; j++)
	{
		cellOffset = j*numCol;
		pointOffset = j*nx;
		for (i=0; i<numCol; i++) 
		{
			if (zc[cellOffset+i] != znull)
			{
				w = (float) (4/sqrt(dx[i]*dx[i] + dy[j]*dy[j]));
				weight[pointOffset+i] += w;
				weight[pointOffset+i+1] += w;
				weight[pointOffset+i+nx] += w;
				weight[pointOffset+i+nx+1] += w;
				v = zc[cellOffset+i]*w;
				value[pointOffset+i] += v;
				value[pointOffset+i+1] += v;
				value[pointOffset+i+nx] += v;
				value[pointOffset+i+nx+1] += v;
			}
		}
	}
    for (i=0; i<nxy; i++)
	{
		if (weight[i] > 0)
		{
			z[i] = value[i]/weight[i];
		}
		else
		{
			z[i] = znull;
		}
	}
	delete value;
	delete weight;
}


void mvUtil::interp3d(float *cellDraw, float *ptDraw, float *dx, float *dy, float *zc,
			  int numCol, int numRow, int numLay, float drawNull, float znull, int *useLayer)
{
	int i, j, k, ic, jc, kc, ip, jp, kp;
	int nx = numCol+1;
	int ny = numRow+1;
	int nz = numLay+1;
	int nxy = nx*ny;
	int nxyz = nx*ny*nz;
	int ncr = numCol*numRow;
	float *value = new float[nxyz];
	float *weight = new float [nxyz];
	float v, w, zz, yy, xx;

	for (i=0; i<nxyz; i++)
	{
		value[i] = 0; 
		weight[i] = 0;
	}

	for (k=0; k<numLay; k++)   // count cell layer
	{
		if (useLayer==0 || (useLayer && useLayer[k]))
		{
			kp = k*nxy;
			kc = k*ncr;
			for (j=0; j<numRow; j++)	// count cell row
			{
				jp = j*nx + kp;
				jc = j*numCol + kc;
				yy = dy[j] * dy[j];
				for (i=0; i<numCol; i++)
				{
					ip = jp + i;
					ic = jc + i;
					xx = dx[i] * dx[i];
					if (zc[ic+ncr] == znull || zc[ic] == znull)
					{
						zz = 0;
					}
					else
					{
						zz = zc[ic+ncr]-zc[ic];
					}
					if (cellDraw[ic]!=drawNull)
					{
						w = (float) (2/sqrt(xx + yy + zz));
						weight[ip] += w;
						weight[ip+1] += w;
						weight[ip+nx] += w;
						weight[ip+nx+1] += w;
						weight[ip+nxy] += w;
						weight[ip+nxy+1] += w;
						weight[ip+nxy+nx] += w;
						weight[ip+nxy+nx+1] += w;
						v = w*cellDraw[ic];
						value[ip] += v;
						value[ip+1] += v;
						value[ip+nx] += v;
						value[ip+nx+1] += v;
						value[ip+nxy] += v;
						value[ip+nxy+1] += v;
						value[ip+nxy+nx] += v;
						value[ip+nxy+nx+1] += v;
					}
				}
			}
		}
	}

	for (i=0; i<nxyz; i++)
	{
		if (weight[i] != 0)	
		{
			ptDraw[i] = value[i]/weight[i];
		}
		else 
		{
			ptDraw[i] = 0;
		}
	}
	delete [] weight;
	delete [] value;

}


int mvUtil::ExtractNumber(ifstream *in, int *buffer)
{
	char line[1024];
	in->getline(line, 1024);
	char *code = strchr(line, '=');
	if (code == 0 || code[1] == '\0')
	{
		return 0;
	}
	*buffer = atoi(code + 2);
	return 1;
}

int mvUtil::ExtractNumber(ifstream *in, float *buffer)
{
	char line[1024];
	in->getline(line, 1024);
	char *code = strchr(line, '=');
	if (code == 0 || code[1] == '\0')
	{
		return 0;
	}
	*buffer = (float) atof(code + 2);
	return 1;
}

int mvUtil::ExtractNumber(ifstream *in, double *buffer)
{
	char line[1024];
	in->getline(line, 1024);
	char *code = strchr(line, '=');
	if (code == 0 || code[1] == '\0')
	{
		return 0;
	}
	*buffer = atof(code + 2);
	return 1;
}

int mvUtil::ExtractString(ifstream *in, char *buffer, int len)
{
	char line[1024];
	in->getline(line, 1024);
	char *code = strchr(line, '=');
	if (code == 0 || code[1] == '\0')
	{
		buffer[0] = '\0';
		return 0;
	}
	strncpy(buffer, code+2, len);
	return 1;
}

int mvUtil::ExtractLabelAndValue(ifstream *in, char *label, char *value)
{
	char line[1024], buffer[1024];
	in->getline(line, 1024);
	char *code = strchr(line, '=');
	if (code == 0 || code[1] == '\0')
	{
		return 0;
	}
	strcpy(buffer, code+1);
	TrimLeft(buffer);
	TrimRight(buffer);
	strcpy(value, buffer);
	*code = '\0';
	TrimRight(line);
	strcpy(label, line);
	return 1;
}

void mvUtil::TrimLeft(char *aString)
{
	if (aString[0] == '\0' || aString[0] != ' ')
	{
		return;
	}
	char *p = aString;
	while (*p == ' ') 
	{
		p++;
		// if the end of the string is reached, then this string contains all spaces
		if (*p == '\0')
		{
			aString[0] = '\0';
			return;
		}
	}

	// shift characters forward
	int lp = strlen(p);
	for (int i=0; i<lp; i++)
	{
		aString[i] = p[i];
	}
	aString[lp] = '\0';
}

void mvUtil::TrimRight(char *aString)
{
	int ls = strlen(aString);
	if (ls == 0 || aString[ls-1] != ' ')
	{
		return;
	}
	char *p = aString + (ls - 1);
	while (*p == ' ')
	{
		p--;
		if (p == aString && *p == ' ')
		{
			aString[0] = '\0';
			return;
		}
	}
	*(p+1) = '\0';
}

void mvUtil::ToLowerCase(char *aString)
{
	for(int i=0; i < (int) strlen(aString); i++ )
	{
		 aString[i] = tolower(aString[i]);
	}
}

float mvUtil::Fortran_atof(char *aString)
{
	TrimLeft(aString);
	TrimRight(aString);
	int StringLength = strlen(aString);
	for (int Index = StringLength-1; Index >= 1; Index--)
	{
		if((aString[Index] == '+') || (aString[Index] == '-'))
		{
			if ((aString[Index-1] != 'e') && (aString[Index-1] != 'E')
				&& (aString[Index-1] != 'd') && (aString[Index-1] != 'D'))
			{
				std::string Source(aString);
				std::string E("E");
				std::string Destination = Source.substr(0,Index);// + E;//  + Source.substr(Index, StringLength-Index);
				Destination += E;
				Destination += Source.substr(Index, StringLength-Index);
				return atof(Destination.c_str());
			}
		}
	}
	return atof(aString);
}


void mvUtil::ExtractFirstString(char *aString, char *FirstString)
{
	TrimLeft(aString);
	int ls = strlen(aString);
	strcpy(FirstString, aString);
	for (int i = 0; i < ls; i++)
	{
		if (aString[i] == ' ')
		{
			aString[i] = '\0';
			FirstString[i] = '\0';
			if (i < ls-1)
			{
				strcpy(aString, aString + i+1);
				//aString = aString + i+1;
			}
			else
			{
				aString[0] = '\0';

			}
			return;

		}
	}
	*aString = 0;
	return;
}
